@@ -20,11 +20,26 @@
 
 from .container_group import ContainerGroup
 from .custom_group import CustomGroup
-from .drillhole_group import DrillholeGroup
+from .drillhole_group import DrillholeGroup, IntegratorDrillholeGroup
 from .giftools_group import GiftoolsGroup
 from .group import Group
 from .group_type import GroupType
+from .integrator_group import (
+    AirborneTheme,
+    EarthModelsTheme,
+    GeochemistryMineralogyDataSet,
+    GeochemistryMineralogyTheme,
+    GeophysicsTheme,
+    GroundTheme,
+    IntegratorGroup,
+    IntegratorProject,
+    ObservationPointsTheme,
+    QueryGroup,
+    RockPropertiesTheme,
+    SamplesTheme,
+)
 from .notype_group import NoTypeGroup
 from .property_group import PropertyGroup
 from .root_group import RootGroup
 from .simpeg_group import SimPEGGroup
+from .survey_group import AirborneGeophysics
