@@ -27,11 +27,12 @@ from typing import TYPE_CHECKING
 import h5py
 import numpy as np
 
-from ..data import CommentsData, Data, DataType, FilenameData, IntegerData
+from ..data import CommentsData, Data, DataType, FilenameData, IntegerData, TextData
 from ..groups import Group, GroupType, RootGroup
 from ..objects import ObjectBase, ObjectType
 from ..shared import Entity, EntityType, fetch_h5_handle
-from .utils import as_str_if_uuid, dict_mapper
+from ..shared.concatenation import Concatenator
+from ..shared.utils import KEY_MAP, as_str_if_uuid, dict_mapper
 
 if TYPE_CHECKING:
     from .. import shared, workspace
@@ -44,19 +45,6 @@ class H5Writer:
 
     str_type = h5py.special_dtype(vlen=str)
 
-    key_map = {
-        "values": "Data",
-        "cells": "Cells",
-        "surveys": "Surveys",
-        "trace": "Trace",
-        "trace_depth": "TraceDepth",
-        "vertices": "Vertices",
-        "octree_cells": "Octree Cells",
-        "property_groups": "PropertyGroups",
-        "color_map": "Color map",
-        "metadata": "Metadata",
-    }
-
     @classmethod
     def create_geoh5(
         cls,
@@ -72,7 +60,7 @@ class H5Writer:
 
         :return h5file: Pointer to a geoh5 file.
         """
-        with fetch_h5_handle(file) as h5file:
+        with fetch_h5_handle(file, mode="r+") as h5file:
             project = h5file.create_group(workspace.name)
             cls.write_attributes(h5file, workspace)
             project.create_group("Data")
@@ -84,7 +72,7 @@ class H5Writer:
             types.create_group("Object types")
 
     @classmethod
-    def create_dataset(cls, entity_handle, dataset: np.ndarray, label: str):
+    def create_dataset(cls, entity_handle, dataset: np.ndarray, label: str) -> None:
         """
         Create a dataset on geoh5.
 
@@ -106,7 +94,7 @@ class H5Writer:
         uid: uuid.UUID,
         ref_type: str,
         parent: Entity,
-    ):
+    ) -> None:
         """
         Remove a child from a parent.
 
@@ -115,11 +103,16 @@ class H5Writer:
         :param ref_type: Input type from: 'Types', 'Groups', 'Objects' or 'Data
         :param parent: Remove entity from parent.
         """
-        with fetch_h5_handle(file) as h5file:
+        with fetch_h5_handle(file, mode="r+") as h5file:
             uid_str = as_str_if_uuid(uid)
             parent_handle = H5Writer.fetch_handle(h5file, parent)
-            if parent_handle is not None and uid_str in parent_handle[ref_type].keys():
+
+            if parent_handle is None or parent_handle.get(ref_type) is None:
+                return
+
+            if uid_str in parent_handle[ref_type]:
                 del parent_handle[ref_type][uid_str]
+                parent.workspace.repack = True
 
     @staticmethod
     def remove_entity(
@@ -127,7 +120,7 @@ class H5Writer:
         uid: uuid.UUID,
         ref_type: str,
         parent: Entity = None,
-    ):
+    ) -> None:
         """
         Remove an entity and its type from the target geoh5 file.
 
@@ -137,17 +130,17 @@ class H5Writer:
         :param parent: Remove entity from parent.
 
         """
-        with fetch_h5_handle(file) as h5file:
-            base = list(h5file.keys())[0]
+        with fetch_h5_handle(file, mode="r+") as h5file:
+            base = list(h5file)[0]
             base_type_handle = h5file[base][ref_type]
             uid_str = as_str_if_uuid(uid)
 
             if ref_type == "Types":
                 for e_type in ["Data types", "Group types", "Object types"]:
-                    if uid_str in base_type_handle[e_type].keys():
+                    if uid_str in base_type_handle[e_type]:
                         del base_type_handle[e_type][uid_str]
             else:
-                if uid_str in base_type_handle.keys():
+                if uid_str in base_type_handle:
                     del base_type_handle[uid_str]
 
                 if parent is not None:
@@ -159,7 +152,7 @@ class H5Writer:
         file: str | h5py.File,
         entity,
         return_parent: bool = False,
-    ):
+    ) -> None | h5py.Group:
         """
         Get a pointer to an :obj:`~geoh5py.shared.entity.Entity` in geoh5.
 
@@ -169,8 +162,8 @@ class H5Writer:
 
         :return entity_handle: HDF5 pointer to an existing entity, parent or None if not found.
         """
-        with fetch_h5_handle(file) as h5file:
-            base = list(h5file.keys())[0]
+        with fetch_h5_handle(file, mode="r+") as h5file:
+            base = list(h5file)[0]
             base_handle = h5file[base]
 
             if entity.name == base:
@@ -201,7 +194,7 @@ class H5Writer:
                     break
 
             # Check if already in the project
-            if as_str_if_uuid(uid) in base_handle.keys():
+            if as_str_if_uuid(uid) in base_handle:
 
                 if return_parent:
                     return base_handle
@@ -210,31 +203,13 @@ class H5Writer:
 
         return None
 
-    @classmethod
-    def finalize(cls, file: str | h5py.File, workspace: workspace.Workspace):
-        """
-        Add/replace the :obj:`~geoh5py.groups.root_group.RootGroup` in geoh5.
-
-        :param file: Name or handle to a geoh5 file
-        :param workspace: Workspace object defining the project structure.
-        """
-        with fetch_h5_handle(file) as h5file:
-            root_handle = cls.save_entity(h5file, workspace.root)
-
-            if "Root" in h5file[workspace.name].keys():
-                del h5file[workspace.name]["Root"]
-            else:
-                h5file[workspace.name].create_group = "Root"
-
-            h5file[workspace.name]["Root"] = root_handle
-
     @classmethod
     def save_entity(
         cls,
         file: str | h5py.File,
         entity,
         add_children: bool = True,
-    ):
+    ) -> h5py.Group:
         """
         Write an :obj:`~geoh5py.shared.entity.Entity` to geoh5 with its
         :obj:`~geoh5py.shared.entity.Entity.children`.
@@ -243,7 +218,7 @@ class H5Writer:
         :param entity: Target :obj:`~geoh5py.shared.entity.Entity`.
         :param add_children: Add :obj:`~geoh5py.shared.entity.Entity.children`.
         """
-        with fetch_h5_handle(file) as h5file:
+        with fetch_h5_handle(file, mode="r+") as h5file:
             new_entity = H5Writer.write_entity(h5file, entity)
 
             if add_children:
@@ -257,61 +232,121 @@ class H5Writer:
         return new_entity
 
     @classmethod
-    def update_attributes(
-        cls,
-        file: str | h5py.File,
-        entity,
-    ):
+    def update_concatenated_field(
+        cls, file: str | h5py.File, entity, attribute: str, channel: str
+    ) -> None:
+        """
+        Update the attributes of a concatenated :obj:`~geoh5py.shared.entity.Entity`.
+
+        :param file: Name or handle to a geoh5 file.
+        :param entity: Target :obj:`~geoh5py.shared.entity.Entity`.
+        :param attribute: Name of the attribute to get updated.
+        :param channel: Name of the data or index to be modified.
+        """
+        with fetch_h5_handle(file, mode="r+") as h5file:
+            entity_handle = H5Writer.fetch_handle(h5file, entity)
+
+            if entity_handle is None:
+                return
+
+            attr_handle = entity_handle["Concatenated Data"].get(attribute.capitalize())
+
+            if attr_handle is None:
+                attr_handle = entity_handle["Concatenated Data"].create_group(
+                    attribute.capitalize()
+                )
+            name = channel.replace("/", "\u2044")
+            try:
+                del attr_handle[name]
+                entity.workspace.repack = True
+            except KeyError:
+                pass
+
+            dict_values = getattr(entity, attribute)
+
+            if channel in dict_values:
+                values = dict_values[channel]
+                if isinstance(values, np.ndarray) and values.dtype == np.float64:
+                    values = values.astype(np.float32)
+
+                if len(values) > 0:
+                    attr_handle.create_dataset(
+                        name,
+                        data=values,
+                        compression="gzip",
+                        compression_opts=9,
+                    )
+
+    @classmethod
+    def update_field(
+        cls, file: str | h5py.File, entity, attribute: str, **kwargs
+    ) -> None:
         """
         Update the attributes of an :obj:`~geoh5py.shared.entity.Entity`.
 
         :param file: Name or handle to a geoh5 file.
         :param entity: Target :obj:`~geoh5py.shared.entity.Entity`.
+        :param attribute: Name of the attribute to get updated.
         """
-        with fetch_h5_handle(file) as h5file:
+        with fetch_h5_handle(file, mode="r+") as h5file:
             entity_handle = H5Writer.fetch_handle(h5file, entity)
 
-            for attr in entity.modified_attributes:
+            if entity_handle is None:
+                return
 
-                try:
-                    del entity_handle[cls.key_map[attr]]
-                    if getattr(entity, attr, None) is None:
-                        continue
-                except KeyError:
-                    pass
-
-                if attr in ["values", "trace_depth", "metadata"]:
-                    cls.write_data_values(h5file, entity, attr)
-                elif attr == "cells":
-                    cls.write_cells(h5file, entity)
-                elif attr in ["surveys", "trace", "vertices"]:
-                    cls.write_coordinates(h5file, entity, attr)
-                elif attr == "octree_cells":
-                    cls.write_octree_cells(h5file, entity)
-                elif attr == "property_groups":
-                    cls.write_property_groups(h5file, entity)
-                elif attr == "cell_delimiters":
-                    cls.write_cell_delimiters(h5file, entity)
-                elif attr == "color_map":
-                    cls.write_color_map(h5file, entity)
-                else:
-                    cls.write_attributes(h5file, entity)
+            if attribute in [
+                "concatenated_attributes",
+                "metadata",
+                "options",
+                "trace_depth",
+                "values",
+            ]:
+                cls.write_data_values(h5file, entity, attribute, **kwargs)
+            elif attribute in [
+                "cells",
+                "concatenated_object_ids",
+                "layers",
+                "octree_cells",
+                "property_group_ids",
+                "prisms",
+                "surveys",
+                "trace",
+                "u_cell_delimiters",
+                "v_cell_delimiters",
+                "vertices",
+                "z_cell_delimiters",
+            ]:
+                cls.write_array_attribute(h5file, entity, attribute, **kwargs)
+            elif attribute == "property_groups":
+                cls.write_property_groups(h5file, entity)
+            elif attribute == "color_map":
+                cls.write_color_map(h5file, entity)
+            elif attribute == "entity_type":
+                del entity_handle["Type"]
+                entity.workspace.repack = True
+                new_type = H5Writer.write_entity_type(h5file, entity.entity_type)
+                entity_handle["Type"] = new_type
+            else:
+                cls.write_attributes(h5file, entity)
 
     @classmethod
     def write_attributes(
         cls,
         file: str | h5py.File,
         entity,
-    ):
+    ) -> None:
         """
         Write attributes of an :obj:`~geoh5py.shared.entity.Entity`.
 
         :param file: Name or handle to a geoh5 file.
         :param entity: Entity with attributes to be added to the geoh5 file.
         """
-        with fetch_h5_handle(file) as h5file:
+        with fetch_h5_handle(file, mode="r+") as h5file:
             entity_handle = H5Writer.fetch_handle(h5file, entity)
 
+            if entity_handle is None:
+                return
+
             for key, attr in entity.attribute_map.items():
 
                 try:
@@ -321,83 +356,36 @@ class H5Writer:
 
                 value = as_str_if_uuid(value)
 
-                if key == "PropertyGroups" or (key == "Metadata" and value is None):
+                if (
+                    key
+                    in [
+                        "PropertyGroups",
+                        "Attributes",
+                        "Property Groups IDs",
+                        "Concatenated object IDs",
+                    ]
+                    or value is None
+                ):  # or key in Concatenator._attribute_map:
                     continue
 
                 if key in ["Association", "Primitive type"]:
-                    value = value.name.lower().capitalize()
+                    value = value.name.lower().title().replace("_", "-")
 
                 if isinstance(value, (np.int8, bool)):
                     entity_handle.attrs.create(key, int(value), dtype="int8")
                 elif isinstance(value, str):
                     entity_handle.attrs.create(key, value, dtype=cls.str_type)
-                elif value is None:
-                    entity_handle.attrs.create(key, "None", dtype=cls.str_type)
                 else:
                     entity_handle.attrs.create(
                         key, value, dtype=np.asarray(value).dtype
                     )
 
-    @classmethod
-    def write_cell_delimiters(
-        cls,
-        file: str | h5py.File,
-        entity,
-    ):
-        """
-        Add cell delimiters (u, v, z)  to a :obj:`~geoh5py.objects.block_model.BlockModel`.
-
-        :param file: Name or handle to a geoh5 file
-        :param entity: Target entity
-        """
-        with fetch_h5_handle(file) as h5file:
-            entity_handle = H5Writer.fetch_handle(h5file, entity)
-
-            if hasattr(entity, "u_cell_delimiters") and (
-                entity.u_cell_delimiters is not None
-            ):
-                cls.create_dataset(
-                    entity_handle, entity.u_cell_delimiters, "U cell delimiters"
-                )
-
-            if hasattr(entity, "v_cell_delimiters") and (
-                entity.v_cell_delimiters is not None
-            ):
-                cls.create_dataset(
-                    entity_handle, entity.v_cell_delimiters, "V cell delimiters"
-                )
-
-            if hasattr(entity, "z_cell_delimiters") and (
-                entity.z_cell_delimiters is not None
-            ):
-                cls.create_dataset(
-                    entity_handle, entity.z_cell_delimiters, "Z cell delimiters"
-                )
-
-    @classmethod
-    def write_cells(
-        cls,
-        file: str | h5py.File,
-        entity,
-    ):
-        """
-        Add :obj:`~geoh5py.objects.object_base.ObjectBase.cells`.
-
-        :param file: Name or handle to a geoh5 file
-        :param entity: Target entity
-        """
-        with fetch_h5_handle(file) as h5file:
-
-            if hasattr(entity, "cells") and (entity.cells is not None):
-                entity_handle = H5Writer.fetch_handle(h5file, entity)
-                cls.create_dataset(entity_handle, entity.cells, "Cells")
-
     @classmethod
     def write_color_map(
         cls,
         file: str | h5py.File,
         entity_type: shared.EntityType,
-    ):
+    ) -> None:
         """
         Add :obj:`~geoh5py.data.color_map.ColorMap` to a
         :obj:`~geoh5py.data.data_type.DataType`.
@@ -405,11 +393,20 @@ class H5Writer:
         :param file: Name or handle to a geoh5 file
         :param entity_type: Target entity_type with color_map
         """
-        with fetch_h5_handle(file) as h5file:
+        with fetch_h5_handle(file, mode="r+") as h5file:
             color_map = getattr(entity_type, "color_map", None)
+            entity_type_handle = H5Writer.fetch_handle(h5file, entity_type)
+
+            if entity_type_handle is None:
+                return
+
+            try:
+                del entity_type_handle["Color map"]
+                entity_type.workspace.repack = True
+            except KeyError:
+                pass
 
             if color_map is not None and color_map.values is not None:
-                entity_type_handle = H5Writer.fetch_handle(h5file, entity_type)
                 cls.create_dataset(
                     entity_type_handle,
                     getattr(color_map, "_values"),
@@ -424,7 +421,7 @@ class H5Writer:
         cls,
         file: str | h5py.File,
         entity_type: shared.EntityType,
-    ):
+    ) -> None:
         """
         Add :obj:`~geoh5py.data.reference_value_map.ReferenceValueMap` to a
         :obj:`~geoh5py.data.data_type.DataType`.
@@ -432,14 +429,23 @@ class H5Writer:
         :param file: Name or handle to a geoh5 file
         :param entity_type: Target entity_type with value_map
         """
-        with fetch_h5_handle(file) as h5file:
+        with fetch_h5_handle(file, mode="r+") as h5file:
             reference_value_map = getattr(entity_type, "value_map", None)
             names = ["Key", "Value"]
             formats = ["<u4", h5py.special_dtype(vlen=str)]
 
-            if reference_value_map is not None and reference_value_map.map is not None:
-                entity_type_handle = H5Writer.fetch_handle(h5file, entity_type)
+            entity_type_handle = H5Writer.fetch_handle(h5file, entity_type)
 
+            if entity_type_handle is None:
+                return
+
+            try:
+                del entity_type_handle["Value map"]
+                entity_type.workspace.repack = True
+            except KeyError:
+                pass
+
+            if reference_value_map is not None and reference_value_map.map is not None:
                 dtype = list(zip(names, formats))
                 array = np.array(list(reference_value_map.map.items()), dtype=dtype)
                 cls.create_dataset(entity_type_handle, array, "Value map")
@@ -449,15 +455,19 @@ class H5Writer:
         cls,
         file: str | h5py.File,
         entity,
-    ):
+    ) -> None:
         """
         Needs revision once Visualization is implemented
 
         :param file: Name or handle to a geoh5 file
         :param entity: Target entity
         """
-        with fetch_h5_handle(file) as h5file:
+        with fetch_h5_handle(file, mode="r+") as h5file:
             entity_handle = H5Writer.fetch_handle(h5file, entity)
+
+            if entity_handle is None:
+                return
+
             dtype = np.dtype(
                 [("ViewID", h5py.special_dtype(vlen=str)), ("Visible", "int8")]
             )
@@ -469,12 +479,9 @@ class H5Writer:
                 visible["Visible"] = 1
 
     @classmethod
-    def write_coordinates(
-        cls,
-        file: str | h5py.File,
-        entity,
-        attribute,
-    ):
+    def write_array_attribute(
+        cls, file: str | h5py.File, entity, attribute, values=None, **kwargs
+    ) -> None:
         """
         Add :obj:`~geoh5py.objects.object_base.ObjectBase.surveys` of an object.
 
@@ -482,27 +489,40 @@ class H5Writer:
         :param entity: Target entity.
         :param attribute: Name of the attribute to be written to geoh5
         """
-        with fetch_h5_handle(file) as h5file:
+        with fetch_h5_handle(file, mode="r+") as h5file:
             entity_handle = H5Writer.fetch_handle(h5file, entity)
 
-            if getattr(entity, attribute, None) is not None:
+            if entity_handle is None:
+                return
+
+            if values is None and getattr(entity, f"{attribute}", None) is not None:
+                values = getattr(entity, f"_{attribute}", None)
+
+            if (
+                isinstance(entity, Concatenator)
+                and attribute != "concatenated_object_ids"
+            ):
+                entity_handle = entity_handle["Concatenated Data"]
+
+            try:
+                del entity_handle[KEY_MAP[attribute]]
+                entity.workspace.repack = True
+            except KeyError:
+                pass
+
+            if values is not None:
                 entity_handle.create_dataset(
-                    cls.key_map[attribute],
-                    data=getattr(entity, "_" + attribute),
+                    KEY_MAP[attribute],
+                    data=values,
                     compression="gzip",
                     compression_opts=9,
+                    **kwargs,
                 )
 
-            elif attribute in entity_handle.keys():
-                del entity_handle[attribute]
-
     @classmethod
     def write_data_values(
-        cls,
-        file: str | h5py.File,
-        entity,
-        attribute,
-    ):
+        cls, file: str | h5py.File, entity, attribute, values=None, **kwargs
+    ) -> None:
         """
         Add data :obj:`~geoh5py.data.data.Data.values`.
 
@@ -510,12 +530,24 @@ class H5Writer:
         :param entity: Target entity.
         :param attribute: Name of the attribute to be written to geoh5
         """
-        with fetch_h5_handle(file) as h5file:
+        with fetch_h5_handle(file, mode="r+") as h5file:
             entity_handle = H5Writer.fetch_handle(h5file, entity)
-            if getattr(entity, attribute, None) is None:
+
+            if entity_handle is None:
                 return
 
-            values = getattr(entity, attribute)
+            if isinstance(entity, Concatenator):
+                entity_handle = entity_handle["Concatenated Data"]
+
+            if KEY_MAP[attribute] in entity_handle:
+                del entity_handle[KEY_MAP[attribute]]
+                entity.workspace.repack = True
+
+            if values is None:
+                if getattr(entity, attribute, None) is None:
+                    return
+
+                values = getattr(entity, "_" + attribute)
 
             # Adding an array of values
             if isinstance(values, dict) or isinstance(entity, CommentsData):
@@ -526,60 +558,75 @@ class H5Writer:
                 values = dict_mapper(values, [as_str_if_uuid])
 
                 entity_handle.create_dataset(
-                    cls.key_map[attribute],
+                    KEY_MAP[attribute],
                     data=json.dumps(values, indent=4),
                     dtype=h5py.special_dtype(vlen=str),
                     shape=(1,),
+                    **kwargs,
                 )
 
             elif isinstance(entity, FilenameData):
-                entity_handle.create_dataset(
-                    "Data",
-                    data=entity.file_name,
-                    dtype=h5py.special_dtype(vlen=str),
-                    shape=(1,),
-                )
-
-                if entity.file_name in entity_handle:
-                    del entity_handle[entity.file_name]
-
-                entity_handle.create_dataset(
-                    entity.file_name,
-                    data=np.asarray(np.void(values[:])),
-                    shape=(1,),
-                )
+                cls.write_file_name_data(entity_handle, entity, values)
 
             elif isinstance(values, str):
                 entity_handle.create_dataset(
-                    cls.key_map[attribute],
+                    KEY_MAP[attribute],
                     data=values,
                     dtype=h5py.special_dtype(vlen=str),
                     shape=(1,),
+                    **kwargs,
                 )
+
             else:
                 out_values = deepcopy(values)
                 if isinstance(entity, IntegerData):
                     out_values = np.round(out_values).astype("int32")
-                else:
-                    out_values[np.isnan(out_values)] = entity.ndv()
+
+                elif isinstance(entity, TextData):
+                    out_values = [val.encode() for val in values]
+
+                if getattr(entity, "ndv", None) is not None:
+                    out_values[np.isnan(out_values)] = entity.ndv
 
                 entity_handle.create_dataset(
-                    cls.key_map[attribute],
+                    KEY_MAP[attribute],
                     data=out_values,
                     compression="gzip",
                     compression_opts=9,
+                    **kwargs,
                 )
-                entity_type_handle = H5Writer.fetch_handle(h5file, entity.entity_type)
-                stats_cache = entity_type_handle.get("StatsCache")
-                if stats_cache is not None:
-                    del entity_type_handle["StatsCache"]
+
+    @classmethod
+    def clear_stats_cache(
+        cls,
+        file: str | h5py.File,
+        entity: Data,
+    ) -> None:
+        """
+        Clear the StatsCache dataset.
+
+        :param file: Name or handle to a geoh5 file.
+        :param entity: Target entity.
+        """
+        with fetch_h5_handle(file, mode="r+") as h5file:
+            if not isinstance(entity, Data):
+                return
+
+            entity_type_handle = H5Writer.fetch_handle(h5file, entity.entity_type)
+            if entity_type_handle is None:
+                return
+
+            stats_cache = entity_type_handle.get("StatsCache")
+            if stats_cache is not None:
+                del entity_type_handle["StatsCache"]
+                entity.workspace.repack = True
 
     @classmethod
     def write_entity(
         cls,
         file: str | h5py.File,
         entity,
-    ):
+    ) -> h5py.Group:
         """
         Add an :obj:`~geoh5py.shared.entity.Entity` and its attributes to geoh5.
         The function returns a pointer to the entity if already present on file.
@@ -589,9 +636,9 @@ class H5Writer:
 
         :return entity: Pointer to the written entity. Active link if "close_file" is False.
         """
-        with fetch_h5_handle(file) as h5file:
+        with fetch_h5_handle(file, mode="r+") as h5file:
 
-            base = list(h5file.keys())[0]
+            base = list(h5file)[0]
 
             if isinstance(entity, Data):
                 entity_type = "Data"
@@ -602,50 +649,42 @@ class H5Writer:
 
             uid = entity.uid
 
-            if entity_type not in h5file[base].keys():
+            if entity_type not in h5file[base]:
                 h5file[base].create_group(entity_type)
 
             # Check if already in the project
-            if as_str_if_uuid(uid) in h5file[base][entity_type].keys():
-
-                if any([entity.modified_attributes]):
-
-                    if "entity_type" in entity.modified_attributes:
-                        entity_handle = cls.fetch_handle(h5file, entity)
-                        del entity_handle["Type"]
-                        new_type = H5Writer.write_entity_type(
-                            h5file, entity.entity_type
-                        )
-                        entity_handle["Type"] = new_type
-
-                    cls.update_attributes(h5file, entity)
-                    entity.modified_attributes = []
-
-                entity.existing_h5_entity = True
+            if as_str_if_uuid(uid) in h5file[base][entity_type]:
+                entity.on_file = True
 
                 return h5file[base][entity_type][as_str_if_uuid(uid)]
 
             entity_handle = h5file[base][entity_type].create_group(as_str_if_uuid(uid))
-
-            if entity_type == "Groups":
+            if isinstance(entity, Concatenator):
+                concat_group = entity_handle.create_group("Concatenated Data")
+                entity_handle.create_group("Data")
+                concat_group.create_group("Index")
+                concat_group.create_group("Data")
+                entity_handle.create_group("Groups")
+            elif entity_type == "Groups":
                 entity_handle.create_group("Data")
                 entity_handle.create_group("Groups")
                 entity_handle.create_group("Objects")
             elif entity_type == "Objects":
                 entity_handle.create_group("Data")
 
-            H5Writer.write_attributes(h5file, entity)
-
-            # Add the type and return a pointer
+            # Add the type
             new_type = H5Writer.write_entity_type(h5file, entity.entity_type)
             entity_handle["Type"] = new_type
-            entity.entity_type.modified_attributes = []
-            entity.entity_type.existing_h5_entity = True
+            entity.entity_type.on_file = True
 
             cls.write_properties(h5file, entity)
+            entity.on_file = True
 
-            entity.modified_attributes = []
-            entity.existing_h5_entity = True
+            if isinstance(entity, RootGroup):
+                if "Root" in h5file[base]:
+                    del h5file[base]["Root"]
+
+                h5file[base]["Root"] = entity_handle
 
         return entity_handle
 
@@ -654,7 +693,7 @@ class H5Writer:
         cls,
         file: str | h5py.File,
         entity_type: shared.EntityType,
-    ):
+    ) -> h5py.Group:
         """
         Add an :obj:`~geoh5py.shared.entity_type.EntityType` to geoh5.
 
@@ -663,8 +702,8 @@ class H5Writer:
 
         :return type: Pointer to :obj:`~geoh5py.shared.entity_type.EntityType` in geoh5.
         """
-        with fetch_h5_handle(file) as h5file:
-            base = list(h5file.keys())[0]
+        with fetch_h5_handle(file, mode="r+") as h5file:
+            base = list(h5file)[0]
             uid = entity_type.uid
 
             if isinstance(entity_type, DataType):
@@ -680,16 +719,11 @@ class H5Writer:
                 h5file[base].create_group("Types")
 
             # Check if already in the project
-            if entity_type_str not in h5file[base]["Types"].keys():
+            if entity_type_str not in h5file[base]["Types"]:
                 h5file[base]["Types"].create_group(entity_type_str)
 
-            if as_str_if_uuid(uid) in h5file[base]["Types"][entity_type_str].keys():
-
-                if any([entity_type.modified_attributes]):
-                    cls.update_attributes(h5file, entity_type)
-                    entity_type.modified_attributes = []
-
-                entity_type.existing_h5_entity = True
+            if as_str_if_uuid(uid) in h5file[base]["Types"][entity_type_str]:
+                entity_type.on_file = True
 
                 return h5file[base]["Types"][entity_type_str][as_str_if_uuid(uid)]
 
@@ -704,72 +738,66 @@ class H5Writer:
             if hasattr(entity_type, "value_map"):
                 H5Writer.write_value_map(h5file, entity_type)
 
-            entity_type.modified_attributes = False
-            entity_type.existing_h5_entity = True
+            entity_type.on_file = True
 
         return new_type
 
     @classmethod
-    def write_octree_cells(
-        cls,
-        file: str | h5py.File,
-        entity,
-    ):
+    def write_file_name_data(
+        cls, entity_handle: h5py.Group, entity: FilenameData, values: bytes
+    ) -> None:
         """
-        Add :obj:`~geoh5py.object.object_base.ObjectBase.cells` of an
-        :obj:`~geoh5py.object.octree.Octree` object to geoh5.
+        Write a dataset for the file name and file blob.
 
-        :param file: Name or handle to a geoh5 file.
-        :param entity: Target entity_type with color_map.
+        :param entity_handle: Pointer to the geoh5 Group.
+        :param entity: Target :obj:`~geoh5py.data.filename_data.FilenameData` entity.
+        :param values: Bytes data
         """
-        with fetch_h5_handle(file) as h5file:
+        if entity.file_name is None:
+            raise AttributeError("FilenameData requires the 'file_name' to be set.")
 
-            if hasattr(entity, "octree_cells") and (entity.octree_cells is not None):
-                entity_handle = H5Writer.fetch_handle(h5file, entity)
-                cls.create_dataset(entity_handle, entity.octree_cells, "Octree Cells")
+        entity_handle.create_dataset(
+            "Data",
+            data=entity.file_name,
+            dtype=h5py.special_dtype(vlen=str),
+            shape=(1,),
+        )
+
+        if entity.file_name in entity_handle:
+            del entity_handle[entity.file_name]
+            entity.workspace.repack = True
+
+        entity_handle.create_dataset(
+            entity.file_name,
+            data=np.asarray(np.void(values[:])),
+            shape=(1,),
+        )
 
     @classmethod
     def write_properties(
         cls,
         file: str | h5py.File,
         entity: Entity,
-    ):
+    ) -> None:
         """
         Add properties of an :obj:`~geoh5py.shared.entity.Entity`.
 
         :param file: Name or handle to a geoh5 file.
         :param entity: Target :obj:`~geoh5py.shared.entity.Entity`.
         """
-        with fetch_h5_handle(file) as h5file:
+        with fetch_h5_handle(file, mode="r+") as h5file:
+            H5Writer.update_field(h5file, entity, "attributes")
 
-            for attribute in ["values", "trace_depth", "metadata"]:
+            for attribute in KEY_MAP:
                 if getattr(entity, attribute, None) is not None:
-                    H5Writer.write_data_values(h5file, entity, attribute)
-
-            if isinstance(entity, ObjectBase) and isinstance(
-                entity.property_groups, list
-            ):
-                H5Writer.write_property_groups(h5file, entity)
-
-            for attribute in ["surveys", "trace", "vertices"]:
-                if getattr(entity, attribute, None) is not None:
-                    H5Writer.write_coordinates(h5file, entity, attribute)
-
-            if getattr(entity, "u_cell_delimiters", None) is not None:
-                H5Writer.write_cell_delimiters(h5file, entity)
-
-            if getattr(entity, "cells", None) is not None:
-                H5Writer.write_cells(h5file, entity)
-
-            if getattr(entity, "octree_cells", None) is not None:
-                H5Writer.write_octree_cells(h5file, entity)
+                    H5Writer.update_field(h5file, entity, attribute)
 
     @classmethod
     def write_property_groups(
         cls,
         file: str | h5py.File,
         entity,
-    ):
+    ) -> None:
         """
         Write :obj:`~geoh5py.groups.property_group.PropertyGroup` associated with
         an :obj:`~geoh5py.shared.entity.Entity`.
@@ -777,24 +805,27 @@ class H5Writer:
         :param file: Name or handle to a geoh5 file.
         :param entity: Target :obj:`~geoh5py.shared.entity.Entity`.
         """
-        with fetch_h5_handle(file) as h5file:
+        with fetch_h5_handle(file, mode="r+") as h5file:
+            entity_handle = H5Writer.fetch_handle(h5file, entity)
+            if entity_handle is None:
+                return
+
+            try:
+                del entity_handle["PropertyGroups"]
+                entity.workspace.repack = True
+            except KeyError:
+                pass
 
             if hasattr(entity, "property_groups") and isinstance(
                 entity.property_groups, list
             ):
-
-                entity_handle = H5Writer.fetch_handle(h5file, entity)
-
-                # Check if a group already exists, then remove and write
-                if "PropertyGroups" in entity_handle.keys():
-                    del entity_handle["PropertyGroups"]
-
                 entity_handle.create_group("PropertyGroups")
                 for p_g in entity.property_groups:
 
                     uid = as_str_if_uuid(p_g.uid)
-                    if uid in entity_handle["PropertyGroups"].keys():
+                    if uid in entity_handle["PropertyGroups"]:
                         del entity_handle["PropertyGroups"][uid]
+                        entity.workspace.repack = True
 
                     entity_handle["PropertyGroups"].create_group(uid)
 
@@ -826,7 +857,7 @@ class H5Writer:
         file: str | h5py.File,
         entity: Entity,
         recursively=False,
-    ):
+    ) -> None:
         """
         Add/create an :obj:`~geoh5py.shared.entity.Entity` and add it to its parent.
 
@@ -835,7 +866,7 @@ class H5Writer:
         :param recursively: Add parents recursively until reaching the
             :obj:`~geoh5py.groups.root_group.RootGroup`.
         """
-        with fetch_h5_handle(file) as h5file:
+        with fetch_h5_handle(file, mode="r+") as h5file:
 
             if isinstance(entity, RootGroup):
                 return
@@ -854,11 +885,11 @@ class H5Writer:
                 return
 
             # Check if child h5py.Group already exists
-            if entity_type not in parent_handle.keys():
+            if entity_type not in parent_handle:
                 parent_handle.create_group(entity_type)
 
             # Check if child uuid not already in h5
-            if as_str_if_uuid(uid) not in parent_handle[entity_type].keys():
+            if as_str_if_uuid(uid) not in parent_handle[entity_type]:
                 parent_handle[entity_type][as_str_if_uuid(uid)] = entity_handle
 
             if recursively:
