@@ -1,4 +1,4 @@
-#  Copyright (c) 2021 Mira Geoscience Ltd.
+#  Copyright (c) 2022 Mira Geoscience Ltd.
 #
 #  This file is part of geoh5py.
 #
@@ -17,12 +17,13 @@
 
 # pylint: disable=unused-import
 # flake8: noqa
-
 from .block_model import BlockModel
 from .curve import Curve
+from .drape_model import DrapeModel
 from .drillhole import Drillhole
 from .geo_image import GeoImage
 from .grid2d import Grid2D
+from .integrator import IntegratorPoints, NeighbourhoodSurface
 from .label import Label
 from .notype_object import NoTypeObject
 from .object_base import ObjectBase
@@ -31,4 +32,10 @@ from .octree import Octree
 from .points import Points
 from .surface import Surface
 from .surveys.direct_current import CurrentElectrode, PotentialElectrode
+from .surveys.electromagnetics.airborne_tem import (
+    AirborneTEMReceivers,
+    AirborneTEMTransmitters,
+)
+from .surveys.electromagnetics.magnetotellurics import MTReceivers
+from .surveys.electromagnetics.tipper import TipperBaseStations, TipperReceivers
 from .surveys.magnetics import AirborneMagnetics
