@@ -15,16 +15,17 @@
 #  You should have received a copy of the GNU Lesser General Public License
 #  along with geoh5py.  If not, see <https://www.gnu.org/licenses/>.
 
-# pylint: disable=R0902
+# pylint: disable=R0902, R0904
 
 from __future__ import annotations
 
+import re
 import uuid
 
 import numpy as np
 
-from ..data.data import Data
-from ..shared.utils import match_values, merge_arrays
+from ..data import Data, FloatData
+from ..shared.utils import merge_arrays
 from .object_base import ObjectType
 from .points import Points
 
@@ -46,6 +47,7 @@ class Drillhole(Points):
             "Cost": "cost",
             "Collar": "collar",
             "Planning": "planning",
+            "End of hole": "end_of_hole",
         }
     )
 
@@ -53,14 +55,13 @@ class Drillhole(Points):
         self._cells: np.ndarray | None = None
         self._collar: np.ndarray | None = None
         self._cost: float | None = 0.0
+        self._depths: FloatData | None = None
+        self._end_of_hole: float | None = None
         self._planning: str = "Default"
         self._surveys: np.ndarray | None = None
         self._trace: np.ndarray | None = None
         self._trace_depth: np.ndarray | None = None
         self._locations = None
-        self._deviation_x = None
-        self._deviation_y = None
-        self._deviation_z = None
         self._default_collocation_distance = 1e-2
 
         super().__init__(object_type, **kwargs)
@@ -76,16 +77,16 @@ class Drillhole(Points):
         Array of indices defining segments connecting vertices.
         """
         if getattr(self, "_cells", None) is None:
-            if self.existing_h5_entity:
-                self._cells = self.workspace.fetch_cells(self.uid)
+            if self.on_file:
+                self._cells = self.workspace.fetch_array_attribute(self)
 
         return self._cells
 
     @cells.setter
     def cells(self, indices):
         assert indices.dtype == "uint32", "Indices array must be of type 'uint32'"
-        self.modified_attributes = "cells"
         self._cells = indices
+        self.workspace.update_attribute(self, "cells")
 
     @property
     def collar(self):
@@ -100,18 +101,24 @@ class Drillhole(Points):
             if isinstance(value, np.ndarray):
                 value = value.tolist()
 
+            if isinstance(value, str):
+                value = [float(n) for n in re.findall(r"\d+\.\d+", value)]
+
             assert len(value) == 3, "Origin must be a list or numpy array of shape (3,)"
 
-            self.modified_attributes = "attributes"
             value = np.asarray(
                 tuple(value), dtype=[("x", float), ("y", float), ("z", float)]
             )
             self._collar = value
+            self.workspace.update_attribute(self, "attributes")
+
         self._locations = None
 
         if self.trace is not None:
-            self.modified_attributes = "trace"
             self._trace = None
+            self._trace_depth = None
+            self.workspace.update_attribute(self, "trace")
+            self.workspace.update_attribute(self, "trace_depth")
 
     @property
     def cost(self):
@@ -121,64 +128,32 @@ class Drillhole(Points):
         return self._cost
 
     @cost.setter
-    def cost(self, value):
-        assert isinstance(value, float), f"Provided cost value must be of type {float}"
+    def cost(self, value: float | int):
+        assert isinstance(
+            value, (float, int)
+        ), f"Provided cost value must be of type {float} or int."
         self._cost = value
+        self.workspace.update_attribute(self, "attributes")
 
     @property
-    def deviation_x(self):
+    def end_of_hole(self) -> float | None:
         """
-        :obj:`numpy.ndarray`: Store the change in x-coordinates along the well path.
+        End of drillhole in meters
         """
-        if getattr(self, "_deviation_x", None) is None and self.surveys is not None:
-            lengths = self.surveys[1:, 0] - self.surveys[:-1, 0]
-            dl_in = np.cos(np.deg2rad(450.0 - self.surveys[:-1, 2] % 360.0)) * np.cos(
-                np.deg2rad(self.surveys[:-1, 1])
-            )
-            dl_out = np.cos(np.deg2rad(450.0 - self.surveys[1:, 2] % 360.0)) * np.cos(
-                np.deg2rad(self.surveys[1:, 1])
-            )
-            ddl = np.divide(dl_out - dl_in, lengths, where=lengths != 0)
-            self._deviation_x = dl_in + lengths * ddl / 2.0
+        return self._end_of_hole
 
-        return self._deviation_x
+    @end_of_hole.setter
+    def end_of_hole(self, value: float | int | None):
+        assert isinstance(
+            value, (int, float, type(None))
+        ), f"Provided end_of_hole value must be of type {int}"
+        self._end_of_hole = value
+        self.workspace.update_attribute(self, "attributes")
 
     @property
-    def deviation_y(self):
+    def locations(self) -> np.ndarray | None:
         """
-        :obj:`numpy.ndarray`: Store the change in y-coordinates along the well path.
-        """
-        if getattr(self, "_deviation_y", None) is None and self.surveys is not None:
-            lengths = self.surveys[1:, 0] - self.surveys[:-1, 0]
-            dl_in = np.sin(np.deg2rad(450.0 - self.surveys[:-1, 2] % 360.0)) * np.cos(
-                np.deg2rad(self.surveys[:-1, 1])
-            )
-            dl_out = np.sin(np.deg2rad(450.0 - self.surveys[1:, 2] % 360.0)) * np.cos(
-                np.deg2rad(self.surveys[1:, 1])
-            )
-            ddl = np.divide(dl_out - dl_in, lengths, where=lengths != 0)
-            self._deviation_y = dl_in + lengths * ddl / 2.0
-
-        return self._deviation_y
-
-    @property
-    def deviation_z(self):
-        """
-        :obj:`numpy.ndarray`: Store the change in z-coordinates along the well path.
-        """
-        if getattr(self, "_deviation_z", None) is None and self.surveys is not None:
-            lengths = self.surveys[1:, 0] - self.surveys[:-1, 0]
-            dl_in = np.sin(np.deg2rad(self.surveys[:-1, 1]))
-            dl_out = np.sin(np.deg2rad(self.surveys[1:, 1]))
-            ddl = np.divide(dl_out - dl_in, lengths, where=lengths != 0)
-            self._deviation_z = dl_in + lengths * ddl / 2.0
-
-        return self._deviation_z
-
-    @property
-    def locations(self):
-        """
-        :obj:`numpy.ndarray`: Lookup array of the well path x,y,z coordinates.
+        Lookup array of the well path in x, y, z coordinates.
         """
         if (
             getattr(self, "_locations", None) is None
@@ -186,37 +161,50 @@ class Drillhole(Points):
             and self.surveys is not None
         ):
             lengths = self.surveys[1:, 0] - self.surveys[:-1, 0]
+            deviation_x = compute_deviation(self.surveys, "x")
+            deviation_y = compute_deviation(self.surveys, "y")
+            deviation_z = compute_deviation(self.surveys, "z")
             self._locations = np.c_[
-                self.collar["x"] + np.cumsum(np.r_[0.0, lengths * self.deviation_x]),
-                self.collar["y"] + np.cumsum(np.r_[0.0, lengths * self.deviation_y]),
-                self.collar["z"] + np.cumsum(np.r_[0.0, lengths * self.deviation_z]),
+                self.collar["x"] + np.cumsum(np.r_[0.0, lengths * deviation_x]),
+                self.collar["y"] + np.cumsum(np.r_[0.0, lengths * deviation_y]),
+                self.collar["z"] + np.cumsum(np.r_[0.0, lengths * deviation_z]),
             ]
 
         return self._locations
 
     @property
-    def planning(self):
+    def planning(self) -> str:
         """
-        :obj:`str`: Status of the hole: ["Default", "Ongoing", "Planned", "Completed"]
+        Status of the hole on of "Default", "Ongoing", "Planned", "Completed" or "No status"
         """
         return self._planning
 
     @planning.setter
-    def planning(self, value):
-        choices = ["Default", "Ongoing", "Planned", "Completed"]
+    def planning(self, value: str):
+        choices = ["Default", "Ongoing", "Planned", "Completed", "No status"]
         assert value in choices, f"Provided planning value must be one of {choices}"
         self._planning = value
+        self.workspace.update_attribute(self, "attributes")
 
     @property
-    def surveys(self):
+    def surveys(self) -> np.ndarray | None:
         """
-        :obj:`numpy.array` of :obj:`float`, shape (3, ): Coordinates of the surveys
+        Coordinates of the surveys
         """
-        if (getattr(self, "_surveys", None) is None) and self.existing_h5_entity:
-            self._surveys = self.workspace.fetch_coordinates(self.uid, "surveys")
+        if (getattr(self, "_surveys", None) is None) and self.on_file:
+            self._surveys = self.workspace.fetch_array_attribute(self, "surveys")
 
-        if getattr(self, "_surveys", None) is not None:
-            surveys = self._surveys.view("<f4").reshape((-1, 3))
+        if isinstance(self._surveys, np.ndarray):
+            try:
+                surveys = self._surveys.view("<f4").reshape((-1, 3))
+            except TypeError:
+                surveys = np.vstack(
+                    [
+                        self._surveys["Depth"],
+                        self._surveys["Azimuth"],
+                        self._surveys["Dip"],
+                    ]
+                ).T
 
             # Repeat first survey point at surface for de-survey interpolation
             surveys = np.vstack([surveys[0, :], surveys])
@@ -234,17 +222,16 @@ class Drillhole(Points):
             if value.shape[1] != 3:
                 raise ValueError("'surveys' requires an ndarray of shape (*, 3)")
 
-            self.modified_attributes = "surveys"
             self._surveys = np.asarray(
                 np.core.records.fromarrays(
-                    value.T, names="Depth, Dip, Azimuth", formats="<f4, <f4, <f4"
+                    value.T, names="Depth, Azimuth, Dip", formats="<f4, <f4, <f4"
                 )
             )
-            self.modified_attributes = "trace"
+            self.workspace.update_attribute(self, "surveys")
+            self.end_of_hole = float(self._surveys["Depth"][-1])
             self._trace = None
-        self._deviation_x = None
-        self._deviation_y = None
-        self._deviation_z = None
+            self.workspace.update_attribute(self, "trace")
+
         self._locations = None
 
     @property
@@ -258,14 +245,15 @@ class Drillhole(Points):
     def default_collocation_distance(self, tol):
         assert tol > 0, "Tolerance value should be >0"
         self._default_collocation_distance = tol
+        self.workspace.update_attribute(self, "attributes")
 
     @property
     def trace(self) -> np.ndarray | None:
         """
         :obj:`numpy.array`: Drillhole trace defining the path in 3D
         """
-        if self._trace is None and self.existing_h5_entity:
-            self._trace = self.workspace.fetch_coordinates(self.uid, "trace")
+        if self._trace is None and self.on_file:
+            self._trace = self.workspace.fetch_array_attribute(self, "trace")
 
         if self._trace is not None:
             return self._trace.view("<f8").reshape((-1, 3))
@@ -284,27 +272,59 @@ class Drillhole(Points):
         return self._trace_depth
 
     @property
-    def _from(self):
+    def from_(self):
+        """
+        Depth data corresponding to the tops of the interval values.
+        """
         data_obj = self.get_data("FROM")
         if data_obj:
             return data_obj[0]
+
         return None
 
     @property
-    def _to(self):
+    def to_(self):
+        """
+        Depth data corresponding to the bottoms of the interval values.
+        """
         data_obj = self.get_data("TO")
         if data_obj:
             return data_obj[0]
+
         return None
 
     @property
-    def _depth(self):
-        data_obj = self.get_data("DEPTH")
-        if data_obj:
-            return data_obj[0]
-        return None
+    def depths(self) -> FloatData | None:
+        if self._depths is None:
+            data_obj = self.get_data("DEPTH")
+            if data_obj and isinstance(data_obj[0], FloatData):
+                self.depths = data_obj[0]
+        return self._depths
 
-    def add_data(self, data: dict, property_group: str = None) -> Data | list[Data]:
+    @depths.setter
+    def depths(self, value: FloatData | np.ndarray | None):
+        if isinstance(value, np.ndarray):
+            value = self.workspace.create_entity(
+                Data,
+                entity={
+                    "parent": self,
+                    "association": "VERTEX",
+                    "name": "DEPTH",
+                    "values": value,
+                },
+                entity_type={"primitive_type": "FLOAT"},
+            )
+
+        if isinstance(value, (FloatData, type(None))):
+            self._depths = value
+        else:
+            raise ValueError(
+                f"Input '_depth' property must be of type{FloatData} or None"
+            )
+
+    def add_data(
+        self, data: dict, property_group: str = None, collocation_distance=None
+    ) -> Data | list[Data]:
         """
         Create :obj:`~geoh5py.data.data.Data` specific to the drillhole object
         from dictionary of name and arguments. A keyword 'depth' or 'from-to'
@@ -326,55 +346,41 @@ class Drillhole(Points):
                     },
             }
 
+        :param property_group: Name of the property group to add the data into.
+
         :return: List of new Data objects.
         """
         data_objects = []
 
-        for name, attr in data.items():
-            assert isinstance(attr, dict), (
+        for name, attributes in data.items():
+            assert isinstance(attributes, dict), (
                 f"Given value to data {name} should of type {dict}. "
-                f"Type {type(attr)} given instead."
+                f"Type {type(attributes)} given instead."
             )
-            assert "values" in list(
-                attr.keys()
-            ), f"Given attr for data {name} should include 'values'"
+            assert (
+                "values" in attributes
+            ), f"Given attributes for data {name} should include 'values'"
 
-            attr["name"] = name
+            attributes["name"] = name
 
-            if "collocation_distance" in attr.keys():
-                assert (
-                    attr["collocation_distance"] > 0
-                ), "Input depth 'collocation_distance' must be >0."
-                collocation_distance = attr["collocation_distance"]
-            else:
-                collocation_distance = self.default_collocation_distance
-
-            if "depth" in attr.keys():
-                attr["association"] = "VERTEX"
-                attr["values"] = self.validate_log_data(
-                    attr["depth"],
-                    attr["values"],
-                    collocation_distance=collocation_distance,
-                )
-                del attr["depth"]
-            elif "from-to" in attr.keys():
-                attr["association"] = "CELL"
-                attr["values"] = self.validate_interval_data(
-                    attr["from-to"],
-                    attr["values"],
-                    collocation_distance=collocation_distance,
-                )
-                del attr["from-to"]
-            else:
-                assert attr["association"] == "OBJECT", (
-                    "Input data dictionary must contain {key:values} "
-                    + "{'depth':numpy.ndarray}, {'from-to':numpy.ndarray} "
-                    + "or {'association': 'OBJECT'}."
+            if attributes["name"] in self.get_data_list():
+                raise UserWarning(
+                    f"Data with name '{attributes['name']}' already present "
+                    f"on the drillhole '{self.name}'. "
+                    "Consider changing the values or renaming."
                 )
 
-            entity_type = self.validate_data_type(attr)
-            kwargs = {"parent": self, "association": attr["association"]}
-            for key, val in attr.items():
+            attributes, new_property_group = self.validate_data(
+                attributes, property_group, collocation_distance=collocation_distance
+            )
+            entity_type = self.validate_data_type(attributes)
+            kwargs = {
+                "name": None,
+                "parent": self,
+                "association": attributes["association"],
+                "allow_move": False,
+            }
+            for key, val in attributes.items():
                 if key in ["parent", "association", "entity_type", "type"]:
                     continue
                 kwargs[key] = val
@@ -383,16 +389,19 @@ class Drillhole(Points):
                 Data, entity=kwargs, entity_type=entity_type
             )
 
-            if property_group is not None:
-                self.add_data_to_group(data_object, property_group)
+            if not isinstance(data_object, Data):
+                continue
+
+            if new_property_group is not None:
+                self.add_data_to_group(data_object, new_property_group)
 
             data_objects.append(data_object)
 
         # Check the depths and re-sort data if necessary
         self.sort_depths()
-        self.workspace.finalize()
+
         if len(data_objects) == 1:
-            return data_object
+            return data_objects[0]
 
         return data_objects
 
@@ -411,14 +420,19 @@ class Drillhole(Points):
             np.searchsorted(self.surveys[:, 0], depths, side="left") - 1,
             0,
         )
-        ind_dev = np.minimum(ind_loc, self.deviation_x.shape[0] - 1)
+
+        deviation_x = compute_deviation(self.surveys, "x")
+        deviation_y = compute_deviation(self.surveys, "y")
+        deviation_z = compute_deviation(self.surveys, "z")
+
+        ind_dev = np.minimum(ind_loc, deviation_x.shape[0] - 1)
         locations = (
             self.locations[ind_loc, :]
             + (depths - self.surveys[ind_loc, 0])[:, None]
             * np.c_[
-                self.deviation_x[ind_dev],
-                self.deviation_y[ind_dev],
-                self.deviation_z[ind_dev],
+                deviation_x[ind_dev],
+                deviation_y[ind_dev],
+                deviation_z[ind_dev],
             ]
         )
         return locations
@@ -436,71 +450,29 @@ class Drillhole(Points):
 
         return indices.astype("uint32")
 
-    def validate_log_data(self, depth, input_values, collocation_distance=1e-4):
-        """
-        Compare new and current depth values, append new vertices if necessary and return
-        an augmented values vector that matches the vertices indexing.
-        """
-        assert len(depth) == len(input_values), (
-            f"Mismatch between input 'depth' shape{depth.shape} "
-            + f"and 'values' shape{input_values.shape}"
-        )
-
-        input_values = np.r_[input_values]
-
-        if self._depth is None:
-            self.workspace.create_entity(
-                Data,
-                entity={
-                    "parent": self,
-                    "association": "VERTEX",
-                    "name": "DEPTH",
-                },
-                entity_type={"primitive_type": "FLOAT"},
-            )
-
-        if self._depth.values is None:  # First data appended
-            self.add_vertices(self.desurvey(depth))
-            depth = np.r_[np.ones(self.n_vertices - depth.shape[0]) * np.nan, depth]
-            values = np.r_[
-                np.ones(self.n_vertices - input_values.shape[0]) * np.nan, input_values
-            ]
-            self._depth.values = depth
-
-        else:
-            depths, indices = merge_arrays(
-                self._depth.values,
-                depth,
-                return_mapping=True,
-                collocation_distance=collocation_distance,
-            )
-            values = merge_arrays(
-                np.ones(self.n_vertices) * np.nan,
-                input_values,
-                replace="B->A",
-                mapping=indices,
-            )
-            self.add_vertices(self.desurvey(np.delete(depth, indices[:, 1])))
-            self._depth.values = depths
-            self.workspace.finalize()
-
-        return values
-
-    def validate_interval_data(self, from_to, input_values, collocation_distance=1e-4):
+    def validate_interval_data(
+        self,
+        from_to: np.ndarray | list,
+        values: np.ndarray,
+        collocation_distance: float = 1e-4,
+    ):
         """
         Compare new and current depth values, append new vertices if necessary and return
         an augmented values vector that matches the vertices indexing.
         """
         if isinstance(from_to, list):
-            from_to = np.vtack(from_to)
+            from_to = np.vstack(from_to)
 
-        assert from_to.shape[0] == len(input_values), (
-            f"Mismatch between input 'from_to' shape{from_to.shape} "
-            + f"and 'values' shape{input_values.shape}"
-        )
-        assert from_to.shape[1] == 2, "The `from-to` values must have shape(*, 2)"
+        if from_to.shape[0] != len(values):
+            raise ValueError(
+                f"Mismatch between input 'from_to' shape{from_to.shape} "
+                + f"and 'values' shape{values.shape}"
+            )
 
-        if (self._from is None) and (self._to is None):
+        if from_to.ndim != 2 or from_to.shape[1] != 2:
+            raise ValueError("The `from-to` values must have shape(*, 2).")
+
+        if (self.from_ is None) and (self.to_ is None):
             uni_depth, inv_map = np.unique(from_to, return_inverse=True)
             self.cells = self.add_vertices(self.desurvey(uni_depth))[inv_map].reshape(
                 (-1, 2)
@@ -525,65 +497,140 @@ class Drillhole(Points):
                 },
                 entity_type={"primitive_type": "FLOAT"},
             )
-        else:
-            from_ind = match_values(
-                self._from.values,
-                from_to[:, 0],
-                collocation_distance=collocation_distance,
-            )
-            to_ind = match_values(
-                self._to.values,
-                from_to[:, 1],
-                collocation_distance=collocation_distance,
-            )
+        elif self.cells is not None and self.from_ is not None and self.to_ is not None:
+            out_vec = np.c_[self.from_.values, self.to_.values]
+            dist_match = []
+            for i, elem in enumerate(from_to):
+                ind = np.where(
+                    np.linalg.norm(elem - out_vec, axis=1) < collocation_distance
+                )[0]
+                if len(ind) > 0:
+                    dist_match.append([ind[0], i])
 
-            # Find matching cells
-            in_match = np.ones((self._from.values.shape[0], 2)) * np.nan
-            in_match[from_ind[:, 0], 0] = from_ind[:, 1]
-            in_match[to_ind[:, 0], 1] = to_ind[:, 1]
-
-            out_match = np.ones_like(from_to) * np.nan
-            out_match[from_ind[:, 1], 0] = from_ind[:, 0]
-            out_match[to_ind[:, 1], 1] = to_ind[:, 0]
-
-            cell_map = np.c_[
-                np.where(in_match[:, 0] == in_match[:, 1])[0],
-                np.where(out_match[:, 0] == out_match[:, 1])[0],
-            ]
+            cell_map: np.ndarray = np.asarray(dist_match, dtype=int)
 
             # Add vertices
             vert_new = np.ones_like(from_to, dtype="bool")
-            vert_new[from_ind[:, 1], 0] = False
-            vert_new[to_ind[:, 1], 1] = False
+            if cell_map.ndim == 2:
+                vert_new[cell_map[:, 1], :] = False
             ind_new = np.where(vert_new.flatten())[0]
             uni_new, inv_map = np.unique(
                 from_to.flatten()[ind_new], return_inverse=True
             )
 
-            # Add cells
-            new_cells = np.ones_like(from_to.flatten()) * np.nan
-            new_cells[ind_new] = self.add_vertices(self.desurvey(uni_new))[inv_map]
-            new_cells = new_cells.reshape((-1, 2))
-            new_cells[from_ind[:, 1], 0] = self.cells[from_ind[:, 0], 0]
-            new_cells[to_ind[:, 1], 1] = self.cells[to_ind[:, 0], 1]
-            new_cells = np.delete(new_cells, cell_map[:, 1], 0)
-
             # Append values
-            input_values = merge_arrays(
+            values = merge_arrays(
                 np.ones(self.n_cells) * np.nan,
-                np.r_[input_values],
+                np.r_[values],
                 replace="B->A",
                 mapping=cell_map,
             )
-            self._from.values = merge_arrays(
-                self._from.values, from_to[:, 0], mapping=cell_map
+            self.cells = np.r_[
+                self.cells,
+                self.add_vertices(self.desurvey(uni_new))[inv_map]
+                .reshape((-1, 2))
+                .astype("uint32"),
+            ]
+            self.from_.values = merge_arrays(
+                self.from_.values, from_to[:, 0], mapping=cell_map
             )
-            self._to.values = merge_arrays(
-                self._to.values, from_to[:, 1], mapping=cell_map
+            self.to_.values = merge_arrays(
+                self.to_.values, from_to[:, 1], mapping=cell_map
             )
-            self.cells = np.r_[self.cells, new_cells.astype("uint32")]
 
-        return input_values
+        return values
+
+    def validate_log_data(
+        self,
+        depth: np.ndarray,
+        input_values: np.ndarray,
+        collocation_distance=1e-4,
+    ) -> np.ndarray:
+        """
+        Compare new and current depth values. Append new vertices if necessary and return
+        an augmented values vector that matches the vertices indexing.
+        """
+        if depth.shape != input_values.shape:
+            raise ValueError(
+                f"Mismatch between input 'depth' shape{depth.shape} "
+                + f"and 'values' shape{input_values.shape}"
+            )
+
+        input_values = np.r_[input_values]
+
+        if self.depths is None:
+            self.add_vertices(self.desurvey(depth))
+            self.depths = np.r_[
+                np.ones(self.n_vertices - depth.shape[0]) * np.nan, depth
+            ]
+            values = np.r_[
+                np.ones(self.n_vertices - input_values.shape[0]) * np.nan, input_values
+            ]
+        else:
+            depths, indices = merge_arrays(
+                self.depths.values,
+                depth,
+                return_mapping=True,
+                collocation_distance=collocation_distance,
+            )
+            values = merge_arrays(
+                np.ones(self.n_vertices) * np.nan,
+                input_values,
+                replace="B->A",
+                mapping=indices,
+            )
+            self.add_vertices(self.desurvey(np.delete(depth, indices[:, 1])))
+            self.depths.values = depths
+
+        return values
+
+    def validate_data(
+        self, attributes: dict, property_group=None, collocation_distance=None
+    ) -> tuple:
+        """
+        Validate input drillhole data attributes.
+
+        :param attributes: Dictionary of data attributes.
+        :param property_group: Input property group to validate against.
+        """
+        if collocation_distance is None:
+            collocation_distance = attributes.get(
+                "collocation_distance", self.default_collocation_distance
+            )
+
+        if collocation_distance < 0:
+            raise UserWarning("Input depth 'collocation_distance' must be >0.")
+
+        if (
+            "depth" not in attributes
+            and "from-to" not in attributes
+            and "association" not in attributes
+        ):
+            assert attributes["association"] == "OBJECT", (
+                "Input data dictionary must contain {key:values} "
+                + "{'from-to':numpy.ndarray} "
+                + "or {'association': 'OBJECT'}."
+            )
+
+        if "depth" in attributes.keys():
+            attributes["association"] = "VERTEX"
+            attributes["values"] = self.validate_log_data(
+                attributes["depth"],
+                attributes["values"],
+                collocation_distance=collocation_distance,
+            )
+            del attributes["depth"]
+
+        if "from-to" in attributes.keys():
+            attributes["association"] = "CELL"
+            attributes["values"] = self.validate_interval_data(
+                attributes["from-to"],
+                attributes["values"],
+                collocation_distance=collocation_distance,
+            )
+            del attributes["from-to"]
+
+        return attributes, property_group
 
     def sort_depths(self):
         """
@@ -606,4 +653,36 @@ class Drillhole(Points):
                     key_map = np.argsort(sort_ind)[self.cells.flatten()]
                     self.cells = key_map.reshape((-1, 2)).astype("uint32")
 
-        self.workspace.finalize()
+
+def compute_deviation(surveys: np.ndarray, axis: str) -> np.ndarray | None:
+    """Compute deviation from survey parameters"""
+    if surveys is None:
+        return None
+
+    lengths = surveys[1:, 0] - surveys[:-1, 0]
+    if axis == "x":
+        dl_in = np.cos(np.deg2rad(450.0 - surveys[:-1, 2] % 360.0)) * np.cos(
+            np.deg2rad(surveys[:-1, 1])
+        )
+        dl_out = np.cos(np.deg2rad(450.0 - surveys[1:, 2] % 360.0)) * np.cos(
+            np.deg2rad(surveys[1:, 1])
+        )
+        ddl = np.divide(dl_out - dl_in, lengths, where=lengths != 0)
+
+    elif axis == "y":
+        dl_in = np.sin(np.deg2rad(450.0 - surveys[:-1, 2] % 360.0)) * np.cos(
+            np.deg2rad(surveys[:-1, 1])
+        )
+        dl_out = np.sin(np.deg2rad(450.0 - surveys[1:, 2] % 360.0)) * np.cos(
+            np.deg2rad(surveys[1:, 1])
+        )
+        ddl = np.divide(dl_out - dl_in, lengths, where=lengths != 0)
+
+    elif axis == "z":
+        dl_in = np.sin(np.deg2rad(surveys[:-1, 1]))
+        dl_out = np.sin(np.deg2rad(surveys[1:, 1]))
+        ddl = np.divide(dl_out - dl_in, lengths, where=lengths != 0)
+
+    else:
+        raise ValueError("Input 'axis' must be 'x', 'y' and 'z'.")
+    return dl_in + lengths * ddl / 2.0
