@@ -68,10 +68,11 @@ class BaseValidator(ABC):
         )
 
     @property
-    @classmethod
     @abstractmethod
-    def validator_type(cls):
-        """Validation type identifier."""
+    def validator_type(self):
+        """
+        Validation type identifier.
+        """
         raise NotImplementedError("Must implement the validator_type property.")
 
 
@@ -80,9 +81,6 @@ class OptionalValidator(BaseValidator):
 
     validator_type = "optional"
 
-    def __init__(self, **kwargs):
-        super().__init__(**kwargs)
-
     @classmethod
     def validate(
         cls,
@@ -104,9 +102,6 @@ class AssociationValidator(BaseValidator):
 
     validator_type = "association"
 
-    def __init__(self, **kwargs):
-        super().__init__(**kwargs)
-
     @classmethod
     def validate(
         cls,
@@ -154,12 +149,10 @@ class PropertyGroupValidator(BaseValidator):
 
     validator_type = "property_group_type"
 
-    def __init__(self, **kwargs):
-        super().__init__(**kwargs)
-
     @classmethod
     def validate(cls, name: str, value: PropertyGroup, valid: str) -> None:
-        if value.property_group_type != valid:
+
+        if (value is not None) and (value.property_group_type != valid):
             raise PropertyGroupValidationError(name, value, valid)
 
 
@@ -167,9 +160,6 @@ class AtLeastOneValidator(BaseValidator):
 
     validator_type = "one_of"
 
-    def __init__(self, **kwargs):
-        super().__init__(**kwargs)
-
     @classmethod
     def validate(cls, name, value, valid):
         if not any(v for v in value.values()):
@@ -183,9 +173,6 @@ class RequiredValidator(BaseValidator):
 
     validator_type = "required"
 
-    def __init__(self, **kwargs):
-        super().__init__(**kwargs)
-
     @classmethod
     def validate(cls, name: str, value: Any, valid: bool) -> None:
         """
@@ -202,9 +189,6 @@ class ShapeValidator(BaseValidator):
 
     validator_type = "shape"
 
-    def __init__(self, **kwargs):
-        super().__init__(**kwargs)
-
     @classmethod
     def validate(cls, name: str, value: Any, valid: tuple[int]) -> None:
         """
@@ -227,9 +211,6 @@ class TypeValidator(BaseValidator):
 
     validator_type = "types"
 
-    def __init__(self, **kwargs):
-        super().__init__(**kwargs)
-
     @classmethod
     def validate(cls, name: str, value: Any, valid: list[type] | type) -> None:
         """
@@ -257,9 +238,6 @@ class UUIDValidator(BaseValidator):
 
     validator_type = "uuid"
 
-    def __init__(self, **kwargs):
-        super().__init__(**kwargs)
-
     @classmethod
     def validate(cls, name: str, value: Any, valid: None = None) -> None:
         """
@@ -282,9 +260,6 @@ class ValueValidator(BaseValidator):
 
     validator_type = "values"
 
-    def __init__(self, **kwargs):
-        super().__init__(**kwargs)
-
     @classmethod
     def validate(cls, name: str, value: Any, valid: list[float | str]) -> None:
         """
