@@ -41,14 +41,27 @@ import numpy as np
 
 from .. import data, groups, objects
 from ..data import CommentsData, Data, DataType
-from ..groups import CustomGroup, DrillholeGroup, Group, PropertyGroup, RootGroup
+from ..groups import (
+    CustomGroup,
+    DrillholeGroup,
+    Group,
+    IntegratorDrillholeGroup,
+    PropertyGroup,
+    RootGroup,
+)
 from ..io import H5Reader, H5Writer
 from ..objects import Drillhole, ObjectBase
 from ..shared import weakref_utils
-from ..shared.concatenation import Concatenated, Concatenator
+from ..shared.concatenation import (
+    Concatenated,
+    ConcatenatedData,
+    ConcatenatedDrillhole,
+    ConcatenatedObject,
+    Concatenator,
+)
 from ..shared.entity import Entity
 from ..shared.exceptions import Geoh5FileClosedError
-from ..shared.utils import as_str_if_utf8_bytes, str2uuid
+from ..shared.utils import as_str_if_utf8_bytes, get_attributes, str2uuid
 
 if TYPE_CHECKING:
     from ..groups import group
@@ -171,7 +184,7 @@ class Workspace(AbstractContextManager):
             self._io_call(H5Writer.save_entity, self.root, add_children=True, mode="r+")
 
         self.geoh5.close()
-
+        self._data = {}
         if self.repack and not isinstance(self.h5file, io.BytesIO):
             temp_file = os.path.join(
                 tempfile.gettempdir(), os.path.basename(self.h5file)
@@ -181,6 +194,7 @@ class Workspace(AbstractContextManager):
                     f'h5repack --native "{self.h5file}" "{temp_file}"',
                     check=True,
                     shell=True,
+                    stdout=subprocess.DEVNULL,
                 )
                 os.remove(self.h5file)
                 shutil.move(temp_file, self.h5file)
@@ -215,24 +229,15 @@ class Workspace(AbstractContextManager):
 
         :return: The Entity registered to the workspace.
         """
-
-        entity_kwargs: dict = {"entity": {"uid": None, "parent": None}}
-        for key in entity.__dict__:
-            if key not in ["_uid", "_entity_type", "_on_file"] + list(omit_list):
-                if key[0] == "_":
-                    key = key[1:]
-
-                entity_kwargs["entity"][key] = getattr(entity, key)
-
-        entity_type_kwargs: dict = {"entity_type": {}}
-        for key in entity.entity_type.__dict__:
-            if key not in ["_workspace", "_on_file"] + list(omit_list):
-                if key[0] == "_":
-                    key = key[1:]
-
-                entity_type_kwargs["entity_type"][key] = getattr(
-                    entity.entity_type, key
-                )
+        entity_kwargs = get_attributes(
+            entity,
+            omit_list=["_uid", "_entity_type", "_on_file"] + list(omit_list),
+            attributes={"uid": None, "parent": None},
+        )
+        entity_type_kwargs = get_attributes(
+            entity.entity_type,
+            omit_list=["_workspace", "_on_file"] + list(omit_list),
+        )
 
         if not isinstance(parent, (ObjectBase, Group, Workspace)):
             raise ValueError(
@@ -244,29 +249,52 @@ class Workspace(AbstractContextManager):
 
         # Assign the same uid if possible
         if parent.workspace.get_entity(entity.uid)[0] is None:
-            entity_kwargs["entity"]["uid"] = entity.uid
+            entity_kwargs["uid"] = entity.uid
 
-        entity_kwargs["entity"]["parent"] = parent
+        entity_kwargs["parent"] = parent
 
         entity_type = type(entity)
         if isinstance(entity, Data):
             entity_type = Data
 
-        if not copy_children and "property_groups" in entity_kwargs["entity"]:
-            del entity_kwargs["entity"]["property_groups"]
+        prop_groups = []
+        if "property_groups" in entity_kwargs:
+            if copy_children:
+                prop_groups = entity_kwargs["property_groups"]
+
+            del entity_kwargs["property_groups"]
 
         new_object = parent.workspace.create_entity(
-            entity_type, **{**entity_kwargs, **entity_type_kwargs}
+            entity_type, **{"entity": entity_kwargs, "entity_type": entity_type_kwargs}
         )
 
         if copy_children:
+            children_map = {}
             for child in entity.children:
-                new_object.add_children(
-                    [self.copy_to_parent(child, new_object, copy_children=True)]
-                )
+                new_child = self.copy_to_parent(child, new_object, copy_children=True)
+                new_object.add_children([new_child])
+                children_map[child.uid] = new_child.uid
+
+            if prop_groups:
+                self.copy_property_groups(new_object, prop_groups, children_map)
+                self.workspace.update_attribute(new_object, "property_groups")
 
         return new_object
 
+    @classmethod
+    def copy_property_groups(
+        cls, entity: ObjectBase, propery_groups: list[PropertyGroup], data_map: dict
+    ):
+        for prop_group in propery_groups:
+            new_group = entity.find_or_create_property_group(
+                **{
+                    "association": prop_group.association,
+                    "name": prop_group.name,
+                    "property_group_type": prop_group.property_group_type,
+                }
+            )
+            new_group.properties = [data_map[uid] for uid in prop_group.properties]
+
     def create_from_concatenation(self, attributes):
         if "Object Type ID" in attributes:
             class_type = ObjectBase
@@ -282,8 +310,9 @@ class Workspace(AbstractContextManager):
             save_on_creation=False,
             **{"entity": attributes, "entity_type": type_attr},
         )
-        recovered_entity.on_file = True
-        recovered_entity.entity_type.on_file = True
+        if recovered_entity is not None:
+            recovered_entity.on_file = True
+            recovered_entity.entity_type.on_file = True
 
         return recovered_entity
 
@@ -318,19 +347,18 @@ class Workspace(AbstractContextManager):
                 and inspect.ismethod(member.primitive_type)
                 and data_type.primitive_type is member.primitive_type()
             ):
-                if (
-                    member is CommentsData
-                    and "UserComments" not in entity_kwargs.values()
+                if member is CommentsData and not any(
+                    isinstance(val, str) and val == "UserComments"
+                    for val in entity_kwargs.values()
                 ):
                     continue
 
                 if self.version > 1.0 and isinstance(
-                    entity_kwargs["parent"], Concatenated
+                    entity_kwargs["parent"], ConcatenatedObject
                 ):
-                    member = type(name + "Concatenated", (Concatenated, member), {})
+                    member = type(name + "Concatenated", (ConcatenatedData, member), {})
 
                 created_entity = member(data_type, **entity_kwargs)
-
                 return created_entity
 
         return None
@@ -411,12 +439,15 @@ class Workspace(AbstractContextManager):
                 and member.default_type_uid() == entity_type_uid
             ):
                 if self.version > 1.0:
-                    if member is DrillholeGroup:
+                    if member in (DrillholeGroup, IntegratorDrillholeGroup):
                         member = type(name + "Concatenator", (Concatenator, member), {})
                     elif member is Drillhole and isinstance(
-                        entity_kwargs.get("parent"), DrillholeGroup
+                        entity_kwargs.get("parent"),
+                        (DrillholeGroup, IntegratorDrillholeGroup),
                     ):
-                        member = type(name + "Concatenated", (Concatenated, member), {})
+                        member = type(
+                            name + "Concatenated", (ConcatenatedDrillhole, member), {}
+                        )
 
                 entity_type = member.find_or_create_type(self, **entity_type_kwargs)
 
@@ -466,33 +497,44 @@ class Workspace(AbstractContextManager):
 
     def remove_children(self, parent, children: list):
         """
-        Remove a list of entities from a parent.
+        Remove a list of entities from a parent. The target entities remain
+        present on file.
         """
         for child in children:
-            if isinstance(child, Data):
-                ref_type = "Data"
+
+            ref_type = self.str_from_type(child)
+
+            if ref_type == "Data":
                 parent.remove_data_from_group(child)
-            elif isinstance(child, Group):
-                ref_type = "Groups"
-            else:
-                ref_type = "Objects"
 
             self._io_call(H5Writer.remove_child, child.uid, ref_type, parent, mode="r+")
 
     def remove_entity(self, entity: Entity):
         """
-        Function to remove an entity and its children from the workspace
+        Function to remove an entity and its children from the workspace.
         """
-        parent = entity.parent
+        if not entity.allow_delete:
+            raise UserWarning(
+                f"The 'allow_delete' property of entity {entity} prevents it from "
+                "being removed. Please revise."
+            )
 
-        self.workspace.remove_recursively(entity)
+        if not isinstance(entity, Concatenator):
+            self.workspace.remove_recursively(entity)
 
-        parent.children.remove(entity)
-
-        del entity
-
-        collect()
-        self.remove_none_referents(self._types, "Types")
+        if isinstance(entity, Concatenated):
+            entity.concatenator.remove_entity(entity)
+        else:
+            ref_type = self.str_from_type(entity)
+            self._io_call(
+                H5Writer.remove_entity,
+                entity.uid,
+                ref_type,
+                mode="r+",
+            )
+            del entity
+            collect()
+            self.remove_none_referents(self._types, "Types")
 
     def remove_none_referents(
         self,
@@ -518,30 +560,9 @@ class Workspace(AbstractContextManager):
         """Delete an entity and its children from the workspace and geoh5 recursively"""
         parent = entity.parent
         for child in entity.children:
-            self.remove_recursively(child)
+            self.remove_entity(child)
 
-        entity.remove_children(entity.children)  # Remove link to children
-
-        if isinstance(entity, Data):
-            ref_type = "Data"
-            parent.remove_data_from_group(entity)
-        elif isinstance(entity, Group):
-            ref_type = "Groups"
-        elif isinstance(entity, ObjectBase):
-            ref_type = "Objects"
-        else:
-            raise NotImplementedError(
-                "Method 'remove_recursively only implemented for classes of type"
-                f" {Data}, {Group} or {ObjectBase}"
-            )
-
-        self._io_call(
-            H5Writer.remove_entity,
-            entity.uid,
-            ref_type,
-            parent=parent,
-            mode="r+",
-        )
+        parent.remove_children([entity])
 
     def deactivate(self):
         """Deactivate this workspace if it was the active one, else does nothing."""
@@ -594,7 +615,7 @@ class Workspace(AbstractContextManager):
 
         :return list: List of children entities.
         """
-        if entity is None or isinstance(entity, Concatenated):
+        if entity is None or isinstance(entity, ConcatenatedData):
             return []
 
         if isinstance(entity, Group):
@@ -647,7 +668,7 @@ class Workspace(AbstractContextManager):
 
     def fetch_concatenated_attributes(self, entity: Group | ObjectBase) -> dict | None:
         """
-        Fetch attributes of Concatenated entities.
+        Fetch attributes of ConcatenatedData entities.
 
         :param entity: Concatenator group.
 
@@ -673,7 +694,7 @@ class Workspace(AbstractContextManager):
         self, entity: Group | ObjectBase, label: str
     ) -> list | None:
         """
-        Fetch list of data or indices of Concatenated entities.
+        Fetch list of data or indices of ConcatenatedData entities.
 
         :param entity: Concatenator group.
         :param label: Label name of the h5py.Group
@@ -700,7 +721,7 @@ class Workspace(AbstractContextManager):
         self, entity: Group | ObjectBase, label: str
     ) -> tuple | None:
         """
-        Fetch data under the Concatenated Data group of an entity.
+        Fetch data under the ConcatenatedData Data group of an entity.
 
         :param entity: Concatenator group.
         :param label: Name of the target data.
@@ -771,7 +792,7 @@ class Workspace(AbstractContextManager):
 
         :return: Array of values.
         """
-        if isinstance(entity, Concatenated):
+        if isinstance(entity, ConcatenatedData):
             return entity.concatenator.fetch_values(entity, entity.name)
 
         return self._io_call(H5Reader.fetch_values, entity.uid)
@@ -916,7 +937,7 @@ class Workspace(AbstractContextManager):
         """
         data_name = {}
         for key, val in self._data.items():
-            entity = val.__call__()
+            entity = val()
             if entity is not None:
                 data_name[key] = entity.name
         return data_name
@@ -938,7 +959,7 @@ class Workspace(AbstractContextManager):
         """
         groups_name = {}
         for key, val in self._groups.items():
-            entity = val.__call__()
+            entity = val()
             if entity is not None:
                 groups_name[key] = entity.name
         return groups_name
@@ -950,7 +971,7 @@ class Workspace(AbstractContextManager):
         """
         objects_name = {}
         for key, val in self._objects.items():
-            entity = val.__call__()
+            entity = val()
             if entity is not None:
                 objects_name[key] = entity.name
         return objects_name
@@ -1095,12 +1116,34 @@ class Workspace(AbstractContextManager):
             entity.concatenator.add_save_concatenated(entity)
 
             if hasattr(entity, "entity_type"):
-                self._io_call(H5Writer.write_entity_type, entity.entity_type, mode="r+")
+                self.save_entity_type(entity.entity_type)
+
         else:
             self._io_call(
                 H5Writer.save_entity, entity, add_children=add_children, mode="r+"
             )
 
+    def save_entity_type(self, entity_type: EntityType) -> None:
+        """
+        Save or update an entity_type to geoh5.
+
+        :param entity_type: Entity to be written to geoh5.
+        """
+        self._io_call(H5Writer.write_entity_type, entity_type, mode="r+")
+
+    @staticmethod
+    def str_from_type(entity) -> str | None:
+        if isinstance(entity, Data):
+            return "Data"
+
+        if isinstance(entity, Group):
+            return "Groups"
+
+        if isinstance(entity, ObjectBase):
+            return "Objects"
+
+        return None
+
     @property
     def types(self) -> list[EntityType]:
         """Get all active entity types registered in the workspace."""
